@@ -1,8 +1,8 @@
 """Top-level package for rframe."""
 
 __author__ = """Yossi Mosbacher"""
-__email__ = 'joe.mosbacher@gmail.com'
-__version__ = '0.1.2'
+__email__ = "joe.mosbacher@gmail.com"
+__version__ = "0.1.3"
 
 
 from .rframe import RemoteFrame
