@@ -1,7 +1,5 @@
-
-
-
 import re
+import pandas as pd
 from typing import Dict, List
 from pydantic import BaseModel
 from pydantic.fields import ModelField, FieldInfo
@@ -9,9 +7,10 @@ from pydantic.fields import ModelField, FieldInfo
 from .indexes import BaseIndex, Index, MultiIndex
 from .indexers import get_indexer
 
+
 def camel_to_snake(name):
-  name = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
-  return re.sub('([a-z0-9])([A-Z])', r'\1_\2', name).lower()
+    name = re.sub("(.)([A-Z][a-z]+)", r"\1_\2", name)
+    return re.sub("([a-z0-9])([A-Z])", r"\1_\2", name).lower()
 
 
 class InsertionError(Exception):
@@ -19,11 +18,11 @@ class InsertionError(Exception):
 
 
 class BaseSchema(BaseModel):
-    _name: str = ''
-    
+    _name: str = ""
+
     def __init_subclass__(cls) -> None:
-        if 'name' not in cls.__dict__:
-            cls.name = camel_to_snake(cls.__name__)
+        if "_name" not in cls.__dict__:
+            cls._name = camel_to_snake(cls.__name__)
         index_fields = cls.get_index_fields()
         indexes = []
         for name, field in index_fields.items():
@@ -39,12 +38,15 @@ class BaseSchema(BaseModel):
         cls.index = index
 
     @classmethod
-    def field_info(cls) -> Dict[str,FieldInfo]:
-        return {name: field.field_info 
-                for name, field in cls.__fields__.items()}
+    def default_datasource(cls):
+        raise NotImplementedError
 
     @classmethod
-    def get_index_fields(cls) -> Dict[str,ModelField]:
+    def field_info(cls) -> Dict[str, FieldInfo]:
+        return {name: field.field_info for name, field in cls.__fields__.items()}
+
+    @classmethod
+    def get_index_fields(cls) -> Dict[str, ModelField]:
         fields = {}
         for name, field in cls.__fields__.items():
             if isinstance(field.field_info, BaseIndex):
@@ -52,7 +54,7 @@ class BaseSchema(BaseModel):
         return fields
 
     @classmethod
-    def get_column_fields(cls) -> Dict[str,ModelField]:
+    def get_column_fields(cls) -> Dict[str, ModelField]:
         fields = {}
         for name, field in cls.__fields__.items():
             if not isinstance(field.field_info, BaseIndex):
@@ -89,7 +91,7 @@ class BaseSchema(BaseModel):
         return {k: data[k] for k in self.get_index_fields()}
 
     @classmethod
-    def compile_query(cls, datastore, **labels)-> List['BaseSchema']:
+    def compile_query(cls, datastore, **labels) -> List["BaseSchema"]:
         indexes = [cls.index_for(name) for name in labels]
         if len(indexes) == 1:
             index = indexes[0]
@@ -99,12 +101,17 @@ class BaseSchema(BaseModel):
             label = labels
 
         label = index.validate_label(label)
+
         indexer = get_indexer(datastore)
-        
+
         return indexer.compile_query(index, label)
 
     @classmethod
-    def find_raw(cls, datastore, **labels)-> List['BaseSchema']:
+    def _find(cls, datastore, **labels) -> List["BaseSchema"]:
+        labels = dict(labels)
+        for name in cls.get_index_fields():
+            if name not in labels:
+                labels[name] = None
         indexes = [cls.index_for(name) for name in labels]
         if len(indexes) == 1:
             index = indexes[0]
@@ -115,25 +122,35 @@ class BaseSchema(BaseModel):
 
         label = index.validate_label(label)
         indexer = get_indexer(datastore)
-        
-        query =  indexer.compile_query(index, label)
+
+        query = indexer.compile_query(index, label)
         docs = query.apply(datastore)
         docs = index.reduce(docs, labels)
         return docs
 
     @classmethod
-    def find(cls, datastore, **labels)-> List['BaseSchema']:
-        docs = cls.find_raw(datastore, **labels)
+    def find(cls, datastore=None, **labels) -> List["BaseSchema"]:
+        if datastore is None:
+            datastore = cls.default_datasource()
+        docs = cls._find(datastore, **labels)
         if not docs:
             return []
-        
+
         docs = [cls(**doc) for doc in docs]
 
         return docs
 
+    @classmethod
+    def find_one(cls, datastore=None, **labels) -> "BaseSchema":
+        docs = cls.find(datastore, **labels)
+        if docs:
+            return docs[0]
+
     @classmethod
     def ensure_index(cls, datastore):
-        pass
+        indexer = get_indexer(datastore)
+        names = list(cls.get_index_fields())
+        return indexer.ensure_index(datastore, names)
 
     def save(self, datastore):
         indexer = get_indexer(datastore)
@@ -149,10 +166,15 @@ class BaseSchema(BaseModel):
 
     def pre_update(self, datastore, new):
         pass
-    
+
     def same_values(self, other):
-        return all([getattr(self, attr) == getattr(other, attr)
-                     for attr in self.get_column_fields()])
+        for attr in self.get_column_fields():
+            left, right = getattr(self, attr), getattr(other, attr)
+            if pd.isna(left) and pd.isna(right):
+                continue
+            if left != right:
+                return False
+        return True
 
     def pandas_dict(self):
         data = self.dict()
